from typing import Iterable, Union

Numerical = Union[float, int]

class TextInput:
    def __init__(self) -> None: ...

    def get_text(self) -> str: ...

    def get_enabled(self) -> bool: ...
    def set_enabled(self, value: bool) -> None: ...

    def clear_text(self) -> None: ...

class FileDropped:
    def get_file_paths(self) -> Iterable[str]: ...
    def get_file_paths_toggle(self) -> Iterable[str]: ...

    def get_file_path_count(self) -> int: ...

    def clear_file_paths(self) -> None: ...

    def get_enabled(self) -> bool: ...
    def set_enabled(self, enabled: bool) -> None: ...