class NoInputDevicesFoundError(Exception): ...

class UnableToReadAudioSampleError(Exception): ...

class PassportNotInitializedError(Exception): ...

class PassportNotRegisteredError(Exception): ...

class DirectoryNotFoundError(Exception): ...

class DirectoryAlreadyExistsError(Exception): ...
