class File:
    def __init__(self, file_path: str) -> None: ...
    def get_absolute_file_path(self) -> None: ...
    def get_relative_file_path(self) -> None: ...
    def is_file_exists(self) -> None: ...
    def move(self, destination: str) -> None: ...
    def copy(self, destination: str) -> None: ...
    def delete(self) -> None: ...
    def recycle(self) -> None: ...
    def rename(self, new_name: str) -> None: ...

class Directory:
    def __init__(self, directory_name: str) -> None: ...
    def get_absolute_path(self) -> None: ...
    def get_relative_path(self) -> None: ...
    def is_directory_exists(self) -> None: ...
    def move(self, destination: str) -> None: ...
    def copy(self, destination: str) -> None: ...
    def delete(self) -> None: ...
    def recycle(self) -> None: ...
    def rename(self, new_name: str) -> None: ...