from pmma.build.NumberFormats import DisplayCoordinate

class Linear:
    start_position: DisplayCoordinate
    end_position: DisplayCoordinate

    def __init__(self, DisplayCoordinateInstance: DisplayCoordinate) -> None: ...
    def start(self) -> None: ...
    def stop(self) -> None: ...
    def pause(self) -> None: ...
    def resume(self) -> None: ...
    def set_duration(self, duration: float) -> None: ...
    def set_looping(self, looping: bool) -> None: ...
    def set_repeating(self, repeating: bool) -> None: ...

    def get_duration(self) -> float: ...
    def get_remaining_duration(self) -> float: ...

    def is_paused(self) -> bool: ...
    def is_playing(self) -> bool: ...
    def is_repeating(self) -> bool: ...
    def is_looping(self) -> bool: ...

class Radial:
    start_position: DisplayCoordinate
    orbit_position: DisplayCoordinate

    def __init__(self, DisplayCoordinateInstance: DisplayCoordinate) -> None: ...
    def start(self) -> None: ...
    def stop(self) -> None: ...
    def pause(self) -> None: ...
    def resume(self) -> None: ...
    def set_duration(self, duration: float) -> None: ...
    def set_repeating(self, repeating: bool) -> None: ...

    def get_duration(self) -> float: ...
    def get_remaining_duration(self) -> float: ...

    def is_paused(self) -> bool: ...
    def is_playing(self) -> bool: ...
    def is_repeating(self) -> bool: ...