from typing import Union, Iterable, Any

class Executor:
    def __init__(self) -> None: ...

    def run(self, command: Union[str, Iterable[str]], blocking: bool=True, hide_window:bool=True) -> None: ...

    def get_exit_code(self) -> int: ...

    def get_result(self) -> Any: ...

class AdvancedExecutor:
    def __init__(self) -> None: ...

    def run(self, command: Union[str, Iterable[str]], hide_window: bool=True) -> None: ...

    def get_busy(self) -> bool: ...

    def get_result(self) -> Any: ...