from typing import Union

Numerical = Union[float, int]

class ButtonPressedEvent:
    def get_pressed(self) -> bool: ...
    def get_pressed_toggle(self) -> bool: ...
    def get_double_pressed(self) -> bool: ...
    def get_long_pressed(self) -> bool: ...

    def poll_long_pressed(self) -> bool: ...

    def get_repeat_press_duration(self) -> Numerical: ...
    def get_long_press_duration(self) -> Numerical: ...
    def get_double_press_duration(self) -> Numerical: ...

    def set_repeat_press_duration(self, duration: Numerical) -> None: ...
    def set_double_press_duration(self, duration: Numerical) -> None: ...
    def set_long_press_duration(self, duration: Numerical) -> None: ...

class Space:
    def __init__(self) -> None: ...

class Apostrophe(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Comma(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Minus(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Period(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Slash(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Zero(ButtonPressedEvent):
    def __init__(self) -> None: ...

class One(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Two(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Three(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Four(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Five(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Six(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Seven(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Eight(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Nine(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Semicolon(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Equal(ButtonPressedEvent):
    def __init__(self) -> None: ...

class A(ButtonPressedEvent):
    def __init__(self) -> None: ...

class B(ButtonPressedEvent):
    def __init__(self) -> None: ...

class C(ButtonPressedEvent):
    def __init__(self) -> None: ...

class D(ButtonPressedEvent):
    def __init__(self) -> None: ...

class E(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F(ButtonPressedEvent):
    def __init__(self) -> None: ...

class G(ButtonPressedEvent):
    def __init__(self) -> None: ...

class H(ButtonPressedEvent):
    def __init__(self) -> None: ...

class I(ButtonPressedEvent):
    def __init__(self) -> None: ...

class J(ButtonPressedEvent):
    def __init__(self) -> None: ...

class K(ButtonPressedEvent):
    def __init__(self) -> None: ...

class L(ButtonPressedEvent):
    def __init__(self) -> None: ...

class M(ButtonPressedEvent):
    def __init__(self) -> None: ...

class N(ButtonPressedEvent):
    def __init__(self) -> None: ...

class O(ButtonPressedEvent):
    def __init__(self) -> None: ...

class P(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Q(ButtonPressedEvent):
    def __init__(self) -> None: ...

class R(ButtonPressedEvent):
    def __init__(self) -> None: ...

class S(ButtonPressedEvent):
    def __init__(self) -> None: ...

class T(ButtonPressedEvent):
    def __init__(self) -> None: ...

class U(ButtonPressedEvent):
    def __init__(self) -> None: ...

class V(ButtonPressedEvent):
    def __init__(self) -> None: ...

class W(ButtonPressedEvent):
    def __init__(self) -> None: ...

class X(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Y(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Z(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Left_Bracket(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Backslash(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Right_Bracket(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Grave_Accent(ButtonPressedEvent):
    def __init__(self) -> None: ...

class World_1(ButtonPressedEvent):
    def __init__(self) -> None: ...

class World_2(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Escape(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Enter(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Tab(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Backspace(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Insert(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Delete(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Right(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Left(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Down(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Up(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Page_Up(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Page_Down(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Home(ButtonPressedEvent):
    def __init__(self) -> None: ...

class End(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Caps_Lock(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Scroll_Lock(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Num_Lock(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Print_Screen(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Pause(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F1(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F2(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F3(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F4(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F5(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F6(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F7(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F8(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F9(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F10(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F11(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F12(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F13(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F14(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F15(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F16(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F17(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F18(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F19(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F20(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F21(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F22(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F23(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F24(ButtonPressedEvent):
    def __init__(self) -> None: ...

class F25(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Left_Shift(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Left_Control(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Left_Alt(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Left_Super(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Right_Shift(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Right_Control(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Right_Alt(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Right_Super(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Shift(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Control(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Alt(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Super(ButtonPressedEvent):
    def __init__(self) -> None: ...

class Menu(ButtonPressedEvent):
    def __init__(self) -> None: ...