class Passport:
    def set_product_name(self, product_name: str) -> None: ...
    def set_product_sub_name(self, product_sub_name: str) -> None: ...
    def set_company_name(self, company_name: str) -> None: ...
    def set_product_version(self, product_version: str) -> None: ...
    def set_product_path(self, product_path: str) -> None: ...
    def set_logging_path(self, logging_path: str) -> None: ...
    def set_profiling_path(self, profiling_path: str) -> None: ...

    def register(self) -> None: ...

    def is_registered(self) -> bool: ...

    def get_product_name(self) -> str: ...
    def get_product_sub_name(self) -> str: ...
    def get_company_name(self) -> str: ...
    def get_product_version(self) -> str: ...
    def get_product_path(self) -> str: ...
    def get_logging_path(self) -> str: ...
    def get_profiling_path(self) -> str: ...