class TimeFormatter:
    def set_from_year(self, year: float) -> None: ...
    def set_from_month(self, month: float) -> None: ...
    def set_from_day(self, day: float) -> None: ...
    def set_from_hour(self, hour: float) -> None: ...
    def set_from_minute(self, minute: float) -> None: ...
    def set_from_second(self, second: float) -> None: ...
    def set_from_microsecond(self, microsecond: float) -> None: ...

    def get_year(self) -> float: ...
    def get_month(self) -> float: ...
    def get_day(self) -> float: ...
    def get_hour(self) -> float: ...
    def get_minute(self) -> float: ...
    def get_second(self) -> float: ...
    def get_microsecond(self) -> float: ...

    def get_in_date_format(self) -> str: ...
    def get_in_sentence_format(self) -> str: ...